// Arduino G-code Interpreter
// v1.0 Carlos guilarte y diego colonnello...
// Modificado para manejar tres ejes en un cnc chimbo...
// Minor modifications by fm to accomodate support for temperature monitoring and LCD displays

#include <Arduino.h>
#include <HardwareSerial.h>
#include "config.h"
#ifdef _LCD_
#include <Wire.h> 
#include <LiquidCrystal_I2C.h>
#endif

#ifdef _TEMP_MON_
#include <thermistor.h>
#include <lm75.h>
#endif

#define LCD_BL_PIN   13
#define FAN_PIN      11
#define ENABLE_PIN   12

// Temperature control constants for fan speed
#define MAX_SPEED      255
#define MAX_TEMP       35.0
#define START_TEMP     25.0
#define TEMP_SPEED_INC (MAX_SPEED/(MAX_TEMP - START_TEMP))

#ifdef _TEMP_MON_
const double FILTER_ALP    = 0.5;

static lm75   mySensor( 0b1001000 );
thermistor    *sensor = NULL;
double filtTemp = 30.0;
#endif

extern  void init_process_string(void);
extern  void init_steppers(void);
extern  void executeInput (void);

#ifdef _LCD_
LiquidCrystal_I2C lcd(0x38);  // Set the LCD I2C address
#endif

#ifdef _TEMP_MON_
void fanControl ( double avrTemp )
{
  int speed = 0;
  
  if ( avrTemp > MAX_TEMP )
  {
    speed = MAX_SPEED;
    analogWrite ( FAN_PIN, speed );
  }
  else if ( avrTemp > START_TEMP )
  {
    speed = (avrTemp - START_TEMP) * TEMP_SPEED_INC;
    analogWrite ( FAN_PIN, speed );
  }
  else
  {
    speed = 0;
    analogWrite ( FAN_PIN, speed );
  }
  lcd.setCursor ( 0,0 );
  lcd.print ( "T:" );
  lcd.print ( avrTemp, 1 );
  lcd.print ("  S:");
  lcd.print ( speed/100 );
  lcd.print ( (speed % 100)/10 );
  lcd.print ( (speed % 10) );
}


void controlTemp ( void )
{
  static int j = 0;
  double currentTemp;  

  currentTemp = sensor->readTemperature();
  filtTemp = ( FILTER_ALP * currentTemp) + (( 1.0 - FILTER_ALP ) * filtTemp);
  //displayTemperature ( currentTemp, filtTemp );
  
  fanControl ( filtTemp );
}
#endif

void setup()
{
  //Do startup stuff here
  Serial.begin(19200);
#ifdef _LCD_
  lcd.begin(16,2);               // initialize the lcd 
#endif
  
  pinMode ( LCD_BL_PIN, OUTPUT );
  pinMode ( FAN_PIN, OUTPUT );
  pinMode ( ENABLE_PIN, OUTPUT );
  digitalWrite ( LCD_BL_PIN, HIGH );
  analogWrite ( FAN_PIN, 150 );
  digitalWrite ( ENABLE_PIN, LOW );

#ifdef _TEMP_MON_
   mySensor.init();
   if ( mySensor.isAvailable() == true )
   {
     sensor = &mySensor;
     filtTemp = sensor->readTemperature();  // Initialise the filter to the first
                                            // reading
   }
#endif

  //other initialization.
  init_process_string();
  init_steppers();

}

void loop()
{
#ifdef _TEMP_MON_  
  static int counter = 0;


  if ( counter == 0 )
  {
    counter=10000;
    controlTemp();
  }
  counter--;
#endif

  //keep it hot!
  executeInput ();

}

